@@ -49,7 +49,7 @@ var TabbrowserPopupController = {
 
 		// JavaScript "window.open()"
 		if (window.open && this.service.isBrowserWindow) {
-			window.__tabextensions_ctrlpopup__open = window.open;
+			window.__tabextensions_ctrlpopup__open = Components.lookupMethod(window, 'open');
 			window.open = eval(this.newWindowOpen.toSource());
 			window.openBrowserTab = this.openBrowserTab;
 			window.shouldOpenBrowserTab = this.shouldOpenBrowserTab;
@@ -66,6 +66,32 @@ var TabbrowserPopupController = {
 			gPopupBlockerObserver.showBlockedPopup = this.showBlockedPopup;
 		}
 
+
+		var events = (this.service.getPref('dom.popup_allowed_events') || '');
+		if (events) {
+			events = events.split(/\s/);
+			if (events.length) {
+				gBrowser.hookEvents = [];
+				for (var i in events)
+					gBrowser.hookEvents.push(events[i]);
+			}
+		}
+/*
+		gBrowser.hookEvents = [
+			'mousedown', 'mouseup', 'click', 'dblclick', 'mouseover',
+			'mouseout', 'mousemove', 'contextmenu', 'keydown', 'keyup', 'keypress',
+			'focus', 'blur', 'load', 'beforeunload', 'unload', 'abort', 'error',
+			'submit', 'reset', 'change', 'select', 'input', 'paint' ,'text',
+			'popupshowing', 'popupshown', 'popuphiding', 'popuphidden', 'close', 'command', 'broadcast', 'commandupdate',
+			'dragenter', 'dragover', 'dragexit', 'dragdrop', 'draggesture', 'resize',
+			'scroll','overflow', 'underflow', 'overflowchanged',
+			'DOMSubtreeModified', 'DOMNodeInserted', 'DOMNodeRemoved',
+			'DOMNodeRemovedFromDocument', 'DOMNodeInsertedIntoDocument',
+			'DOMAttrModified', 'DOMCharacterDataModified',
+			'popupBlocked'
+		];
+*/
+
 	},
  
 	onAfterInit : function() 
@@ -93,30 +119,26 @@ var TabbrowserPopupController = {
  
 	onTabLoading : function(aRelatedTab, aWindow, aShouldNotFollowFrames) 
 	{
-		var frames;
-		try {
-			frames = Components.lookupMethod(aWindow, 'frames').call(aWindow);
-		}
-		catch(e) {
-		}
+		if (aWindow == '[object XULElement]') return;
+
+		var winWrapper = new XPCNativeWrapper(aWindow,
+				'frames',
+				'open()'
+			);
+		var frames = winWrapper.frames;
 		if (frames && !aShouldNotFollowFrames) {
 			for (var i = 0; i < frames.length; i++)
 				this.onTabLoading(aRelatedTab, frames[i]);
 		}
 
-		if (
-			'__tabextensions_ctrlpopup__initialized' in aWindow ||
-			!aWindow.open ||
-			!aWindow.Window
-			)
+		if ('__tabextensions_ctrlpopup__initialized' in aWindow)
 			return;
 
 		// override "window.open"
-		if (!('__tabextensions_ctrlpopup__open' in aWindow.Window.prototype)) {
-			aWindow.Window.prototype.__tabextensions_ctrlpopup__open = aWindow.Window.prototype.open;
+		if (!('__tabextensions_ctrlpopup__open' in aWindow)) {
+			aWindow.__tabextensions_ctrlpopup__open = Components.lookupMethod(aWindow, 'open');
 		}
-		aWindow.eval('Window.prototype.open = '+this.newWindowOpen.toSource());
-		aWindow.open = aWindow.Window.prototype.open;
+		eval('aWindow.open = '+this.newWindowOpen.toSource());
 
 /* THIS SECTION MAKES A HUGE SECURITY HOLE!! (delayed scripts in webpages will be able to access XPCOM without the special permission)
 		// override timer functions
@@ -124,15 +146,15 @@ var TabbrowserPopupController = {
 			aWindow.Window.prototype.__tabextensions_ctrlpopup__setTimeout = aWindow.Window.prototype.setTimeout;
 			aWindow.Window.prototype.__tabextensions_ctrlpopup__setInterval = aWindow.Window.prototype.setInterval;
 		}
-		aWindow.eval('Window.prototype.setTimeout = '+this.newWindowSetTimeout.toSource());
-		aWindow.eval('Window.prototype.setInterval = '+this.newWindowSetInterval.toSource());
+		eval('aWindow.setTimeout = '+this.newWindowSetTimeout.toSource());
+		eval('aWindow.setInterval = '+this.newWindowSetInterval.toSource());
 
 		aWindow.__tabextensions_ctrlpopup__popupControlState   = this.openAbused;
 		aWindow.__tabextensions_ctrlpopup__runningTimeoutDepth = 0;
 */
 
 		// Norton Internet Securityt overrides window.open so we have to kill the feature...
-		aWindow.SymRealWinOpen = aWindow.Window.prototype.open;
+		aWindow.SymRealWinOpen = winWrapper.open;
 
 
 		aWindow.openBrowserTab                  = this.openBrowserTab;
@@ -164,12 +186,15 @@ var TabbrowserPopupController = {
 		var TPC = TabbrowserPopupController;
 if (TS.debug) dump('newWindowOpen ('+aURI+')\n');
 
-		var contextWindow = this.document.defaultView;
+		var contextWindow = TS.getWindowFromDocument(this.document, window);
 		if (!contextWindow) contextWindow = this;
+
+		var topWin = Components.lookupMethod(contextWindow, 'top').call(contextWindow);
+
 		var sourceURI = ('Components' in window && 'lookupMethod' in Components) ? Components.lookupMethod(contextWindow, 'location').call(contextWindow).href : contextWindow.location.href ;
 
 		aURI = aURI ? String(aURI) : 'about:blank' ;
-		if (!aURI.match(/^\w+:/))
+		if (aURI.search(/^\w+:/) < 0)
 			aURI = TS.makeURLAbsolute(sourceURI, aURI);
 
 		var abuseLevel = TPC.checkForAbusePoint({
@@ -188,9 +213,9 @@ if (TS.debug) dump('   blocked\n');
 				true,
 				false,
 				{
-					window    : contextWindow,
-					uri       : aURI,
-					flags     : aFlags
+					window : contextWindow,
+					uri    : aURI,
+					flags  : aFlags
 				}
 			);
 			return {};
@@ -199,22 +224,55 @@ if (TS.debug) dump('   blocked\n');
 try {
 if (TS.debug) dump('   POP UP NOW!!\n');
 		var w;
+		if (
+			aName &&
+			aName.search(/^_(top|self|content|parent)$/i) < 0 &&
+			aName != '_main' &&
+			(w = TS.getFrameByName(topWin, aName))
+			) {
+if (TS.debug) dump('    => Existing Frame\n');
+			TS.uriSecurityCheck(aURI, sourceURI);
+			var docShell = Components.lookupMethod(w, 'QueryInterface').call(w, Components.interfaces.nsIInterfaceRequestor)
+					.getInterface(Components.interfaces.nsIWebNavigation)
+					.QueryInterface(Components.interfaces.nsIDocShell);
+
+
+			// block cross-site scriptings!
+			if (
+				aURI.indexOf('javascript:') == 0 &&
+				docShell &&
+				TS.makeURIFromSpec(sourceURI).host != docShell.QueryInterface(Components.interfaces.nsIWebNavigation).currentURI.host
+				)
+				// see http://lxr.mozilla.org/mozilla/source/dom/src/jsurl/nsJSProtocolHandler.cpp#228
+				throw 'Attempt to load a javascript: URL from one host\nin a window displaying content from another host\nwas blocked by the security manager.';
+
+
+			docShell.loadURI(
+				aURI,
+				Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE,
+				TS.makeURIFromSpec(sourceURI),
+				null,
+				null
+			);
+			return  Components.lookupMethod(docShell.document, 'defaultView').call(docShell.document);
+		}
+
 		if (window.shouldOpenBrowserTab(aURI, aName, aFlags)) {
 if (TS.debug) dump('    => New Tab\n');
 			w = TPC.openBrowserTabInternal(contextWindow, aURI, aName, aFlags, sourceURI);
 		}
 		else {
 if (TS.debug) dump('    => New Window\n');
-			var topWin = Components.lookupMethod(contextWindow, 'top').call(contextWindow);
 			if (
-				topWin.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+				topWin
+					.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
 					.getInterface(Components.interfaces.nsIWebNavigation)
 					.QueryInterface(Components.interfaces.nsIDocShell)
 					.QueryInterface(Components.interfaces.nsIDocShellTreeItem)
 					.itemType != Components.interfaces.nsIDocShellTreeItem.typeChrome &&
 				aName &&
 				!(
-					aName.match(/^_(top|self|content|parent)$/i) ||
+					aName.search(/^_(top|self|content|parent)$/i) > -1 ||
 					aName == '_main'
 				) &&
 				TS.getPref('browser.tabs.extensions.ignore_target') &&
@@ -229,20 +287,34 @@ if (TS.debug) dump('    => New Window\n');
 			if (originalVal)
 				TS.setPref('dom.disable_open_during_load', false);
 
+			TS.setPref('browser.tabs.extensions.processing_window_count_from_script', (TS.getPref('browser.tabs.extensions.processing_window_count_from_script') || 0)+1);
+
 			try {
-			w = (aFlags !== void(0)) ? this.__tabextensions_ctrlpopup__open(aURI, aName, (Components.lookupMethod(this.document.defaultView, 'top').call(this.document.defaultView) == Components.lookupMethod(window, 'top').call(window) ? flags.replace(/(alwaysLowered|alwaysRaised|titlebar|z-lock)(=(0|1|true|false|yes|no))?,?/g, '') || void(0) : flags )) :
-					this.__tabextensions_ctrlpopup__open(aURI, aName) ;
+				w = (aFlags !== void(0)) ?
+					this.__tabextensions_ctrlpopup__open.call(this,
+						String(aURI),
+						String(aName),
+						(topWin == Components.lookupMethod(window, 'top').call(window) ?
+							(flags.replace(/(alwaysLowered|alwaysRaised|titlebar|z-lock)(=(0|1|true|false|yes|no))?,?/g, '') || void(0))
+							: flags
+						)
+					) :
+					this.__tabextensions_ctrlpopup__open.call(this, String(aURI), String(aName)) ;
 			}
 			catch(e) {
 				if (originalVal)
 					TS.setPref('dom.disable_open_during_load', true);
 
+				TS.setPref('browser.tabs.extensions.processing_window_count_from_script', Math.max((TS.getPref('browser.tabs.extensions.processing_window_count_from_script') || 0)-1, 0));
+
 				throw e;
 			}
 
 			if (originalVal)
 				TS.setPref('dom.disable_open_during_load', true);
 
+			TS.setPref('browser.tabs.extensions.processing_window_count_from_script', Math.max((TS.getPref('browser.tabs.extensions.processing_window_count_from_script') || 0)-1, 0));
+
 			// set real opener to prevent to reopen the new window in a tab
 			if (TS.browserWindow &&
 				TS.browserWindow != window)
@@ -441,7 +513,7 @@ if (TS.debug) dump('   allowed\n');
 			if (
 				aName &&
 				(
-					aName.match(/^_(top|self|content)$/i) ||
+					aName.search(/^_(top|self|content)$/i) > -1 ||
 					aName == '_main' ||
 					this.service.WindowWatcher.getWindowByName(aName, aCurrentWindow)
 				)
@@ -582,8 +654,18 @@ if (TS.debug) dump('   allowed\n');
 			}
 			--w.__tabextensions_ctrlpopup__runningTimeoutDepth;
 			delete w.__tabextensions_ctrlpopup__popupState;
+
+			delete w;
+			delete exp;
+			delete popupState;
+			delete runTimeout;
 		};
 
+		delete delay;
+		delete args;
+		delete TS;
+		delete TPC;
+
 		return eval('this.__tabextensions_ctrlpopup__setTimeout(runTimeout, delay)');
 	},
  
@@ -631,10 +713,10 @@ if (TS.debug) dump('   allowed\n');
   
 	openBrowserTab : function(aURI, aName, aFlags, aSourceURI) 
 	{
-		var contextWindow = this.document.defaultView;
+		var TS  = window.TabbrowserService;
+		var contextWindow = TS.getWindowFromDocument(this.document, this);
 		if (!contextWindow) contextWindow = this;
 
-		var TS  = window.TabbrowserService;
 		var TPC = window.TabbrowserPopupController;
 if (TS.debug) dump('newWindowOpen/openBrowserTab ('+aURI+')\n');
 
@@ -707,7 +789,7 @@ if (TS.debug) dump('   allowed\n');
 		// prevent to open same URI
 		if (this.service.preventSameURLTab) {
 			if (b.selectedTab.getAttribute('tab-loadingURI') == uri)
-				return b.selectedTab.mBrowser.contentDocument.defaultView;
+				return b.selectedTab.mBrowser.contentWindow;
 
 			for (i = 0; i < b.mTabs.length; i++)
 			{
@@ -720,13 +802,13 @@ if (TS.debug) dump('   allowed\n');
 					b.scrollTabbarTo(b.selectedTab);
 					b.setFocusInternal();
 				}
-				return b.mTabs[i].mBrowser.contentDocument.defaultView;
+				return b.mTabs[i].mBrowser.contentWindow;
 			}
 		}
 
 		var topWin = Components.lookupMethod(aContextWindow, 'top').call(aContextWindow);
 		for (i = 0; i < b.mTabs.length; i++)
-			if (b.mTabs[i].mBrowser.contentDocument.defaultView == topWin)
+			if (b.mTabs[i].mBrowser.contentWindow == topWin)
 				parentTab = b.mTabs[i];
 
 		var referrer = !parentTab || parentTab.referrerBlocked ? null : this.service.makeURIFromSpec(aSourceURI) ;
@@ -736,12 +818,17 @@ if (TS.debug) dump('   allowed\n');
 			!this.service.getPref('browser.tabs.extensions.ignore_target') &&
 			name.toLowerCase() != '_blank'
 			) {
+			var contentWindow;
 			for (i = 0; i < b.mTabs.length; i++)
 			{
 				browser = b.mTabs[i].mBrowser;
+				contentWindow = new XPCNativeWrapper(browser.contentWindow,
+					'opener',
+					'name'
+				);
 
 				if (b.mTabs[i].browserName != name &&
-					browser.contentDocument.defaultView.name != name) continue;
+					contentWindow.name != name) continue;
 
 				var realURI = this.service.getRealURI(uri);
 				var bypassReferrerBlocker = realURI ? true : false ;
@@ -765,17 +852,17 @@ if (TS.debug) dump('   allowed\n');
 
 				if (b.tabGroupsAvailable) b.attachTabTo(b.mTabs[i], parentTab, true);
 
-				browser.contentDocument.defaultView.opener = aContextWindow;
+				contentWindow.opener = aContextWindow;
 				if (
 					name &&
-					!String(name).match(/^_(self|top|parent|content|blank)$/i) &&
+					String(name).search(/^_(self|top|parent|content|blank)$/i) < 0 &&
 					name != '_main'
 					)
-					browser.contentDocument.defaultView.name = name;
+					contentWindow.name = name;
 
 	//			dump('tabextensions:window.open is redirected to existing tab:'+uri+'\n');
 
-				return browser.contentDocument.defaultView;
+				return browser.contentWindow;
 			}
 		}
 
@@ -813,6 +900,7 @@ if (TS.debug) dump('   allowed\n');
 					info[props[i]] = parentTab[props[i]];
 				info.uri = uri;
 			}
+			delete props;
 		}
 
 		var openIn = this.service.getPref('browser.tabs.extensions.open_tab_in_link');
@@ -832,16 +920,42 @@ if (TS.debug) dump('   allowed\n');
 			b.setFocusInternal();
 		}
 
-		w = t.mBrowser.contentDocument.defaultView;
+		w = new XPCNativeWrapper(t.mBrowser.contentWindow,
+				'opener',
+				'name'
+			);
 
 		w.opener = aContextWindow;
 		if (
 			name &&
-			!String(name).match(/^_(self|top|parent|content|blank)$/i) &&
+			String(name).search(/^_(self|top|parent|content|blank)$/i) < 0 &&
 			name != '_main'
 			)
 			w.name = name;
 
+		var dummyFunc = function() {};
+
+		if (!w.resizeTo) w.resizeTo = dummyFunc;
+		if (!w.resizeBy) w.resizeBy = dummyFunc;
+		if (!w.moveTo)   w.moveTo   = dummyFunc;
+		if (!w.moveBy)   w.moveBy   = dummyFunc;
+		if (!w.focus)    w.focus    = dummyFunc;
+		if (!w.location) w.location = { href : uri };
+
+		delete uri;
+		delete event;
+		delete i;
+		delete name;
+		delete b;
+		delete browser;
+		delete parentTab;
+		delete loadInBackground;
+		delete topWin;
+		delete referrer;
+		delete info;
+		delete openIn;
+		delete t;
+
 		return w;
 	},
 	
@@ -898,7 +1012,7 @@ if (TS.debug) dump('   allowed\n');
 			case 2:
 				if (
 					aFlags &&
-		//			String(aFlags).match(/(^|,)(width|height)=[0-9]*/i) &&
+		//			String(aFlags).search(/(^|,)(width|height)=[0-9]*/i) > -1 &&
 					window.TabbrowserService.winHookMode != 2
 					)
 					return false;
@@ -909,7 +1023,7 @@ if (TS.debug) dump('   allowed\n');
 		return (
 			(
 				!inContent &&
-				(!aURI || !aURI.match(/^chrome:/)) && // not-registered resources
+				(!aURI || aURI.indexOf('chrome:') != 0) && // not-registered resources
 				(
 					// unrequested open in single window mode
 					window.TabbrowserService.winHookMode == 2 ||
@@ -983,4 +1097,4 @@ if (!window.TabbrowserServiceModules)
 if (TabbrowserPopupController.enabled)
 	TabbrowserServiceModules.push(TabbrowserPopupController);
 }
- 
+ 
\ No newline at end of file
