@@ -158,6 +158,10 @@ pref("browser.tabs.extensions.focus_with_selected_order",      false);
 
 
 pref("browser.tabs.extensions.show_progress",        true);
+pref("browser.tabs.extensions.highlight_current",     false);
+pref("browser.tabs.extensions.highlight_current.style", "");
+pref("browser.tabs.extensions.highlight_unread",     false);
+pref("browser.tabs.extensions.highlight_unread.style", "italic text-red");
 // -1 = none, 0 = always, 1 = only for current, 2 = only for pointed
 pref("browser.tabs.extensions.show_closebox.tab",       -1);
 pref("browser.tabs.extensions.show_closebox.tab.delay", 500);
@@ -178,11 +182,15 @@ pref("browser.tabs.extensions.open_tab_in",          3);
 pref("browser.tabs.extensions.open_tab_in_link",     -1);
 // if true, the behavior of tab dragging/dropping goes back to the old behavior.
 pref("browser.tabs.extensions.dragdrop.only_load_uri", false);
+pref("browser.tabs.extensions.dragdrop.indicator",     'default');
 
 // same to "browser.tabs.extensions.tabbar_on***"
 pref("browser.tabs.extensions.ondblclick",    0);
 pref("browser.tabs.extensions.onmiddleclick", 4);
 pref("browser.tabs.extensions.onrightclick",  -1);
+pref("browser.tabs.extensions.tabFlip",       false);
+pref("browser.tabs.extensions.tabFlip.delay", 250);
+pref("browser.tabs.extensions.tabFlip.roop",  false);
 
 pref("browser.tabs.extensions.slow_down_autoreload_in_background.enabled", false);
 pref("browser.tabs.extensions.slow_down_autoreload_in_background.rate", 5);
