@@ -90,10 +90,11 @@ function cancel()
 function onCommand(aEvent)
 {
 	var node = aEvent.target;
+
 	switch (node.localName)
 	{
 		case 'checkbox':
-			gData[node.id].newValue = node.checked;
+			updateValue(node.id, node.checked);
 			break;
 
 		case 'radio':
@@ -101,10 +102,17 @@ function onCommand(aEvent)
 				node = node.parentNode;
 		case 'radiogroup':
 		case 'textbox':
-			gData[node.id].newValue = node.value;
+			updateValue(node.id, node.value);
 			break;
 	}
+}
 
+function updateValue(aID, aValue)
+{
+	if (!(aID in gData) || !gData[aID])
+		return;
+
+	gData[aID].newValue = aValue;
 	gModified = true;
 }
 
